---
title: Objects
---

<script lang="ts">
import InstancedMeshSimple from '$examples/InstancedMeshSimple.svelte'
import ThrelteWrapper from '$examples/ThrelteWrapper.svelte'
</script>

# Objects

## \<Mesh>

#### Example

```svelte
<script>
  import { Mesh } from 'threlte'
  import { BoxBufferGeometry, MeshBasicMaterial } from 'three'
</script>

<Mesh
  position={{ y: 1 }}
  geometry={new BoxBufferGeometry(1, 2, 1)}
  material={new MeshBasicMaterial({ wireframe: true })}
/>
```

#### Properties

```ts
// required
geometry: THREE.BufferGeometry
material: THREE.Material | THREE.Material[]

// optional
position: Position | undefined = undefined
scale: Scale | undefined = undefined
rotation: Rotation | undefined = undefined
lookAt: LookAt | undefined = undefined
viewportAware: boolean = false
castShadow: boolean | undefined = undefined
receiveShadow: boolean | undefined = undefined
frustumCulled: boolean | undefined = undefined
renderOrder: number | undefined = undefined
visible: boolean | undefined = undefined
interactive: boolean = false
ignorePointer: boolean = false
```

#### Bindings

```ts
inViewport: boolean
mesh: THREE.Mesh
```

#### Events

```ts
viewportenter: undefined
viewportleave: undefined
click: CustomEvent<ThreltePointerEvent>
contextmenu: CustomEvent<ThreltePointerEvent>
pointerup: CustomEvent<ThreltePointerEvent>
pointerdown: CustomEvent<ThreltePointerEvent>
pointerenter: CustomEvent<ThreltePointerEvent>
pointerleave: CustomEvent<ThreltePointerEvent>
pointermove: CustomEvent<ThreltePointerEvent>
```

---

## \<InstancedMesh>

The `<InstancedMesh>` is a special version of [`<Mesh>`](/docs/components/objects#mesh) with instanced rendering support. Use `<InstancedMesh>` if you have to render a large number of objects with the same geometry and material but with different world transformations and colors. The usage of `<InstancedMesh>` will help you to reduce the number of draw calls and thus improve the overall rendering performance in your application.

#### Usage

An `<InstancedMesh>` is used in conjunction with the [`<Instance>`](#instance) component:

```svelte
<InstancedMesh {geometry} {material}>
  <Instance />
  <Instance />
</InstancedMesh>
```

It's also possible to nest other objects in an `<InstancedMesh>` component:

```svelte
<InstancedMesh {geometry} {material}>
  <Instance />
  <Instance />

  <GLTF />
</InstancedMesh>
```

Provide an `id` to use multiple `<InstancedMesh>` components:

```svelte
<InstancedMesh id="tree" geometry={treeGeometry} material={treeMaterial}>
  <InstancedMesh id="leaf" geometry={leafGeometry} material={leafMaterial}>
    <Group position={{ x: 1 }}>
      <Instance id="tree" /> // Instance of InstancedMesh with id="tree"
      <Instance id="leaf" /> // Instance of InstancedMesh with id="leaf"
    </Group>

    <Group position={{ x: -2 }}>
      <Instance id="tree" />
      <Instance id="leaf" />
    </Group>
  </InstancedMesh>
</InstancedMesh>
```

#### Instance count

Usually you don't have to worry to set the [property count](https://threejs.org/docs/index.html?q=insta#api/en/objects/InstancedMesh.count) of the underlying `THREE.InstancedMesh` as the component `<InstancedMesh>` automatically scales with the nested `<Instance>` components. In some occasions (e.g. when the amount of instances changes frequently) it might be beneficial to explicitly set the _maximum_ amount of instances:

```svelte
<InstancedMesh count={10} {geometry} {material}>
  <Instance />
  <Instance />
</InstancedMesh>
```

#### Events

Instances also supports DOM-like events like the `<Mesh>` component, the setup however is a bit different. Although you can listen to events on `<Instance>` components, `interactive` and `ignorePointer` need to be set on the `<InstancedMesh>` component:

```svelte
<InstancedMesh interactive {geometry} {material}>
  <Instance on:click={onClick} />
</InstancedMesh>
```

:::admonition type="note"
This means you can't exclude one instance from raycasting while listening for events on another.
:::

#### Nesting

Instances can be nested in other objects and all parent transformations apply as usual:

```svelte
<InstancedMesh interactive {geometry} {material}>
  <Group rotation={{ z: DEG2RAD * 180 }}>
    <Instance />

    <Group position={{ y: 2 }}>
      <Instance />
    </Group>
  </Group>
</InstancedMesh>
```

:::admonition type="info"
Nesting instances might have a slight performance hit, the frameloop will run continuously. Use with caution.
:::

#### Example

@[code](../../../examples/InstancedMeshSimple.svelte)

<ThrelteWrapper>
  <InstancedMeshSimple />
</ThrelteWrapper>

#### Properties

```ts
// required
geometry: THREE.BufferGeometry
material: THREE.Material | THREE.Material[]


// optional
count: number | undefined = undefined
id: string = ''
position: Position | undefined = undefined
scale: Scale | undefined = undefined
rotation: Rotation | undefined = undefined
lookAt: LookAt | undefined = undefined
viewportAware: boolean = false
castShadow: boolean | undefined = undefined
receiveShadow: boolean | undefined = undefined
renderOrder: number | undefined = undefined
visible: boolean | undefined = undefined
interactive: boolean = false
ignorePointer: boolean = false
```

#### Bindings

```ts
instancedMesh: Writable<InstancedMesh>
inViewport: boolean
```

#### Events

```ts
viewportenter: undefined
viewportleave: undefined
```

---

## \<Instance>

Every `<Instance>` component nested in an [`<InstancedMesh>`](#instancedmesh) component resembles one instance. An `<Instance>` can therefore only be used as a child component to a `<InstancedMesh>` component. The `<Instance>` component can be transformed and colorized individually:

```svelte
<InstancedMesh {geometry} {material}>
  <Instance position={{ x: 5 }} scale={1} />
  <Instance position={{ z: 2 }} scale={2} color="red" />
</InstancedMesh>
```

The properties `position`, `rotation` and `scale` work just like with every other threlte component, `color` is passed on to the [THREE.Color](https://threejs.org/docs/#api/en/math/Color) constructor.

Provide an `id` to use multiple `<InstancedMesh>` components:

```svelte
<InstancedMesh id="tree" geometry={treeGeometry} material={treeMaterial}>
  <InstancedMesh id="leaf" geometry={leafGeometry} material={leafMaterial}>
    <Group position={{ x: 1 }}>
      <Instance id="tree" /> // Instance of InstancedMesh with id="tree"
      <Instance id="leaf" /> // Instance of InstancedMesh with id="leaf"
    </Group>

    <Group position={{ x: -2 }}>
      <Instance id="tree" />
      <Instance id="leaf" />
    </Group>
  </InstancedMesh>
</InstancedMesh>
```

---

## \<Group>

#### Example

```svelte
<script>
  import { Group, GLTF } from 'threlte'
</script>

<Group position={{ x: 5 }}>
  <GLTF url={'/models/modelA.glb'} />
  <GLTF url={'/models/modelB.glb'} />
</Group>
```

#### Properties

```ts
// optional
id: string = ''
position: Position | undefined = undefined
scale: Scale | undefined = undefined
rotation: Rotation | undefined = undefined
lookAt: LookAt | undefined = undefined
viewportAware: boolean = false
castShadow: boolean | undefined = undefined
receiveShadow: boolean | undefined = undefined
frustumCulled: boolean | undefined = undefined
renderOrder: number | undefined = undefined
visible: boolean | undefined = undefined
```

#### Bindings

```ts
inViewport: boolean
group: THREE.Group
```

#### Events

```ts
viewportenter: undefined
viewportleave: undefined
```

---

## \<Object3D>

#### Example

You might want to use this component to pass as a reference to other components:

```svelte
<script>
  import { Object3D, PerspectiveCamera } from 'threlte'

  let target
</script>

<PerspectiveCamera lookAt={target} />
<Object3D bind:object={target} />
```

#### Properties

```ts
// optional
position: Position | undefined = undefined
scale: Scale | undefined = undefined
rotation: Rotation | undefined = undefined
lookAt: LookAt | undefined = undefined
viewportAware: boolean = false
castShadow: boolean | undefined = undefined
receiveShadow: boolean | undefined = undefined
frustumCulled: boolean | undefined = undefined
renderOrder: number | undefined = undefined
visible: boolean | undefined = undefined
```

#### Bindings

```ts
inViewport: boolean
object: THREE.Object3D
```

#### Events

```ts
viewportenter: undefined
viewportleave: undefined
```

---

## \<GLTF>

To use DRACO compression, provide a path to the DRACO decoder.  
To use KTX2 compressed textures, provide a path to the KTX2 transcoder.

You are able to change the property `url` to load new 3D content. New content will be swapped as soon as loading is finished.

The `<GLTF>` component supports interaction:

```svelte
<GLTF
  url="/models/helmet/DamagedHelmet.gltf"
  interactive
  on:click={() => {
    console.log('User clicked!')
  }}
/>
```

:::admonition type="tip"
See [this discussion](https://github.com/grischaerbe/threlte/discussions/60#discussioncomment-2746125) on how to use animations of glTF Files.
:::

#### Example

```svelte
<script>
  import { GLTF } from 'threlte'
</script>

<GLTF castShadow receiveShadow url={'/models/flower.glb'} position={{ y: 1 }} scale={3} />
```

#### Properties

```ts
// required
url: string

// optional
position: Position | undefined = undefined
scale: Scale | undefined = undefined
rotation: Rotation | undefined = undefined
lookAt: LookAt | undefined = undefined
viewportAware: boolean = false
castShadow: boolean | undefined = undefined
receiveShadow: boolean | undefined = undefined
frustumCulled: boolean | undefined = undefined
renderOrder: number | undefined = undefined
visible: boolean | undefined = undefined
dracoDecoderPath: string | undefined = undefined
ktxTranscoderPath: string | undefined = undefined
ignorePointer: boolean = false
interactive: boolean = false
```

#### Bindings

```ts
gltf: GLTF | undefined
scene: THREE.Group | undefined
animations: AnimationClip[] | undefined
asset:
  | {
      copyright?: string | undefined;
      generator?: string | undefined;
      version?: string | undefined;
      minVersion?: string | undefined;
      extensions?: any;
      extras?: any;
    }
  | undefined
cameras: Camera[] | undefined
scenes: Group[] | undefined
userData: any
parser: GLTFParser | undefined
```

#### Events

```ts
load: GLTF // The content finished loading
unload: undefined // New content finished loading and the old content is unloaded and disposed
error: string // An error occured while loading and parsing
viewportenter: undefined
viewportleave: undefined
click: CustomEvent<ThreltePointerEvent>
contextmenu: CustomEvent<ThreltePointerEvent>
pointerup: CustomEvent<ThreltePointerEvent>
pointerdown: CustomEvent<ThreltePointerEvent>
pointerenter: CustomEvent<ThreltePointerEvent>
pointerleave: CustomEvent<ThreltePointerEvent>
pointermove: CustomEvent<ThreltePointerEvent>
```
