---
title: Object Instances
---

# Object Instances

While components like `<Mesh>` and `<Group>` create a new object for you (in the case of `<Mesh>` it's a `THREE.Mesh`), an object instance component accepts an existing object instance (`THREE.Mesh`, `THREE.Object3D`, `THREE.Light` or `THREE.Camera`) as a property and applies reactivity to it. It's used internally but can also be used to introduce reactivity to objects that need to be instanced manually as well as imported models or objects that did not yet make it into threlte.

#### Example <!-- omit in toc -->

```svelte
<script>
  import { Object3D } from 'three'
  import { Object3DInstance, useFrame } from 'threlte'

  const object = new Object3D()
  let position = { x: 1, y: 1, z: 1 }

  useFrame(() => {
    position.x += 0.1
  })
</script>

<Object3DInstance {object} {position} />
```

## \<Object3DInstance>

#### Example <!-- omit in toc -->

```svelte
<script>
  import { Object3D } from 'three'
  import { Object3DInstance } from 'threlte'

  const object = new Object3D()
</script>

<Object3DInstance {object} position={{ y: 1 }} />
```

#### Properties <!-- omit in toc -->

```ts
// required
object: THREE.Object3D

// optional
viewportAware: boolean = false
position: Position | undefined = undefined
scale: Scale | undefined = undefined
rotation: Rotation | undefined = undefined
lookAt: LookAt | undefined = undefined
castShadow: boolean | undefined = undefined
receiveShadow: boolean | undefined = undefined
frustumCulled: boolean | undefined = undefined
renderOrder: number | undefined = undefined
visible: boolean | undefined = undefined
```

#### Bindings <!-- omit in toc -->

```ts
inViewport: boolean
```

#### Events <!-- omit in toc -->

```ts
viewportenter: undefined
viewportleave: undefined
```

## \<MeshInstance>

#### Example <!-- omit in toc -->

```svelte
<script>
  import { Mesh, MeshStandardMaterial, BoxBufferGeometry } from 'three'
  import { MeshInstance } from 'threlte'

  const material = new MeshStandardMaterial({
    wireframe: true,
    color: 'black'
  })
  const geometry = new BoxBufferGeometry(1, 1, 1)
  const mesh = new Mesh(geometry, material)
</script>

<MeshInstance {mesh} rotation={{ x: 90 * (Math.PI / 180) }} />
```

#### Properties <!-- omit in toc -->

```ts
// required
mesh: THREE.Mesh

// optional
interactive: boolean = false
ignorePointer: boolean = false
viewportAware: boolean = false
position: Position | undefined = undefined
scale: Scale | undefined = undefined
rotation: Rotation | undefined = undefined
lookAt: LookAt | undefined = undefined
castShadow: boolean | undefined = undefined
receiveShadow: boolean | undefined = undefined
frustumCulled: boolean | undefined = undefined
renderOrder: number | undefined = undefined
visible: boolean | undefined = undefined
```

#### Bindings <!-- omit in toc -->

```ts
inViewport: boolean
```

#### Events <!-- omit in toc -->

```ts
viewportenter: undefined
viewportleave: undefined
click: CustomEvent<ThreltePointerEvent>
contextmenu: CustomEvent<ThreltePointerEvent>
pointerup: CustomEvent<ThreltePointerEvent>
pointerdown: CustomEvent<ThreltePointerEvent>
pointerenter: CustomEvent<ThreltePointerEvent>
pointerleave: CustomEvent<ThreltePointerEvent>
pointermove: CustomEvent<ThreltePointerEvent>
```

## \<CameraInstance>

#### Example <!-- omit in toc -->

```svelte
<script>
  import { PerspectiveCamera } from 'three'
  import { CameraInstance } from 'threlte'

  const camera = new PerspectiveCamera(45, 1, 1, 1000)
</script>

<CameraInstance useCamera={false} {camera} />
```

#### Properties <!-- omit in toc -->

```ts
// required
camera: THREE.Camera

// optional
viewportAware: boolean = false
useCamera: boolean = false
position: Position | undefined = undefined
scale: Scale | undefined = undefined
rotation: Rotation | undefined = undefined
lookAt: LookAt | undefined = undefined
castShadow: boolean | undefined = undefined
receiveShadow: boolean | undefined = undefined
frustumCulled: boolean | undefined = undefined
renderOrder: number | undefined = undefined
visible: boolean | undefined = undefined
```

#### Bindings <!-- omit in toc -->

```ts
inViewport: boolean
```

#### Events <!-- omit in toc -->

```ts
viewportenter: undefined
viewportleave: undefined
```

## \<LightInstance>

#### Example <!-- omit in toc -->

```svelte
<script>
  import { RectAreaLight } from 'three'
  import { RectAreaLightUniformsLib } from 'three/examples/jsm/lights/RectAreaLightUniformsLib'
  import { LightInstance } from 'threlte'

  RectAreaLightUniformsLib.init()

  export let intensity

  const light = new RectAreaLight(0xffffff, intensity, 10, 10)
</script>

<LightInstance {light} {intensity} position={{ x: 5, y: 5 }} />
```

#### Properties <!-- omit in toc -->

```ts
// required
light: THREE.Light

// optional
viewportAware: boolean = false
position: Position | undefined = undefined
scale: Scale | undefined = undefined
rotation: Rotation | undefined = undefined
lookAt: LookAt | undefined = undefined
castShadow: boolean | undefined = undefined
receiveShadow: boolean | undefined = undefined
frustumCulled: boolean | undefined = undefined
renderOrder: number | undefined = undefined
visible: boolean | undefined = undefined
color: THREE.ColorRepresentation | undefined = undefined
intensity: number | undefined = undefined
```

#### Bindings <!-- omit in toc -->

```ts
inViewport: boolean
```

#### Events <!-- omit in toc -->

```ts
viewportenter: undefined
viewportleave: undefined
```

## \<AudioInstance>

#### Example <!-- omit in toc -->

```svelte
<script>
  import { AudioInstance, useAudioListener } from 'threlte'
  import { Audio } from 'three'

  export let buffer: AudioBuffer

  const { listener } = useAudioListener()
  const audio = new Audio(listener)
  audio.setBuffer(buffer)
</script>

<AudioInstance {audio} />
```

#### Properties <!-- omit in toc -->

```ts
// required
audio: THREE.Audio | THREE.PositionalAudio

// optional
viewportAware: boolean = false
position: Position | undefined = undefined
scale: Scale | undefined = undefined
rotation: Rotation | undefined = undefined
lookAt: LookAt | undefined = undefined
castShadow: boolean | undefined = undefined
receiveShadow: boolean | undefined = undefined
frustumCulled: boolean | undefined = undefined
renderOrder: number | undefined = undefined
visible: boolean | undefined = undefined
autoplay: boolean | undefined = undefined
detune: number | undefined = undefined
source: string | AudioBuffer | HTMLMediaElement | AudioBufferSourceNode | MediaStream | undefined = undefined
volume: number | undefined = undefined
loop: boolean | undefined = undefined
filters: BiquadFilterNode[] | BiquadFilterNode | undefined = undefined
playbackRate: number | undefined = undefined
id: string | undefined = undefined
```

#### Bindings <!-- omit in toc -->

```ts
inViewport: boolean
play: (delay?: number) => Promise<THREE.Audio | THREE.PositionalAudio>
pause: () => THREE.Audio | THREE.PositionalAudio
stop: () => THREE.Audio | THREE.PositionalAudio
```

#### Events <!-- omit in toc -->

```ts
viewportenter: undefined
viewportleave: undefined
load: AudioBuffer
progress: ProgressEvent<EventTarget>
error: ErrorEvent
```
